%option 8bit yylineno noyywrap
%option stack

%option noyyget_lineno
%option noyyget_in
%option noyyget_out
%option noyyget_text
%option noyyget_debug
%option noyyget_leng

%option noyy_top_state
%option noyy_scan_buffer
%option noyy_scan_bytes
%option noyy_scan_string

%option reentrant bison-bridge
%option prefix="Qyy"

%option extra-type="struct Qcontext_t *"

true		[Tt][Rr][Uu][Ee]
false		[Ff][Aa][Ll][Ss][Ee]
null		[Nn][Uu][Ll][Ll]
integer		[-+]?[0-9]+
float		([-+]?([0-9]*)?\.[0-9]*([eE][-+]?[0-9]+)?)|([-+]?([0-9]+)(\.[0-9]*)?[eE][-+]?[0-9]+)
hexchar		[0-9a-fA-F]{4}
ident		[a-zA-Z_]+[0-9a-zA-Z_]*

%{
    #include <stdlib.h>

    #include "qf.h"
#define MAX(a, b)   ((a>b)?(a):(b))
#define TEXT_BLOCK_SIZE  1024
struct Qcontext_t
{
    char   *value;
    size_t  used;
    size_t  size;
};
#ifdef	NOTYET
static char * Qdonate(struct Qcontext_t *tok);
static void   Qappend(struct Qcontext_t *tok, char *text);
#endif

    #include "Qgrammar.h"
    void yyerror(char *);
%}

O	[0-7]
D	[0-9]
NZ	[1-9]
L	[a-zA-Z_]
A	[a-zA-Z_0-9]
H	[a-fA-F0-9]
HP	(0[xX])
E	([Ee][+-]?{D}+)
P	([Pp][+-]?{D}+)
FS	(f|F|l|L)
IS	(((u|U)(l|L|ll|LL)?)|((l|L|ll|LL)(u|U)?))
CP	(u|U|L)
SP	(u8|u|U|L)
ES	(\\(['"\?\\abfnrtv]|[0-7]{1,3}|x[a-fA-F0-9]+))
WS	[ \t\v\n\f\r]

%%
	/* NOTYET
	"/*"		{ comment(); }
	 */
"#".*		{ /* consume #-comment */ }
"//".*		{ /* consume //-comment */ }

	/* NOTYET
	"auto"					{ return(AUTO); }
	"break"					{ return(BREAK); }
	"case"					{ return(CASE); }
	"char"					{ return(CHAR); }
	"const"					{ return(CONST); }
	"continue"				{ return(CONTINUE); }
	"default"				{ return(DEFAULT); }
	"do"					{ return(DO); }
	"double"				{ return(DOUBLE); }
	"else"					{ return(ELSE); }
	"enum"					{ return(ENUM); }
	"extern"				{ return(EXTERN); }
	"float"					{ return(FLOAT); }
	"for"					{ return(FOR); }
	"goto"					{ return(GOTO); }
	"if"					{ return(IF); }
	"inline"				{ return(INLINE); }
	"int"					{ return(INT); }
	"long"					{ return(LONG); }
	"register"				{ return(REGISTER); }
	"restrict"				{ return(RESTRICT); }
	"return"				{ return(RETURN); }
	"short"					{ return(SHORT); }
	"signed"				{ return(SIGNED); }
	"sizeof"				{ return(SIZEOF); }
	"static"				{ return(STATIC); }
	"struct"				{ return(STRUCT); }
	"switch"				{ return(SWITCH); }
	"typedef"				{ return(TYPEDEF); }
	"union"					{ return(UNION); }
	"unsigned"				{ return(UNSIGNED); }
	"void"					{ return(VOID); }
	"volatile"				{ return(VOLATILE); }
	"while"					{ return(WHILE); }
	"_Alignas"                              { return ALIGNAS; }
	"_Alignof"                              { return ALIGNOF; }
	"_Atomic"                               { return ATOMIC; }
	"_Bool"                                 { return BOOL; }
	"_Complex"                              { return COMPLEX; }
	"_Generic"                              { return GENERIC; }
	"_Imaginary"                            { return IMAGINARY; }
	"_Noreturn"                             { return NORETURN; }
	"_Static_assert"                        { return STATIC_ASSERT; }
	"_Thread_local"                         { return THREAD_LOCAL; }
	"__func__"                              { return FUNC_NAME; }
	 */
"else"		{ return ELSE; }
"if"		{ return IF; }
"while"		{ return WHILE; }

"print"		{ return PRINT; }

	/* NOTYET
	{L}{A}*					{ return check_type(); }
	 */

[a-z]		{ yylval->sIndex = *yytext - 'a'; return VARIABLE; }

{HP}{H}+{IS}?	{ yylval->I = atoll(yytext); return I_CONSTANT; }
{NZ}{D}*{IS}?	{ yylval->I = atoll(yytext); return I_CONSTANT; }
"0"{O}*{IS}?	{ yylval->I = atoll(yytext); return I_CONSTANT; }
	/* NOTNOW
	0		{ yylval->I = atoll(yytext); return I_CONSTANT; }
	[1-9][0-9]*	{ yylval->I = atoll(yytext); return I_CONSTANT; }
	 */

	/* NOTYET
	{CP}?"'"([^'\\\n]|{ES})+"'"		{ return I_CONSTANT; }
	 */

	/* NOTYET
	{D}+{E}{FS}?				{ return F_CONSTANT; }
	{D}*"."{D}+{E}?{FS}?			{ return F_CONSTANT; }
	{D}+"."{E}?{FS}?			{ return F_CONSTANT; }
	{HP}{H}+{P}{FS}?			{ return F_CONSTANT; }
	{HP}{H}*"."{H}+{P}{FS}?			{ return F_CONSTANT; }
	{HP}{H}+"."{P}{FS}?			{ return F_CONSTANT; }
	 */

"**"		{ return POW; }

	/* NOTYET
	({SP}?\"([^"\\\n]|{ES})*\"{WS}*)+	{ return STRING_LITERAL; }
	"..."					{ return ELLIPSIS; }
	">>="					{ return RIGHT_ASSIGN; }
	"<<="					{ return LEFT_ASSIGN; }
	"+="					{ return ADD_ASSIGN; }
	"-="					{ return SUB_ASSIGN; }
	"*="					{ return MUL_ASSIGN; }
	"/="					{ return DIV_ASSIGN; }
	"%="					{ return MOD_ASSIGN; }
	"&="					{ return AND_ASSIGN; }
	"^="					{ return XOR_ASSIGN; }
	"|="					{ return OR_ASSIGN; }
	 */

">>"		{ return RSHIFT; }
"<<"		{ return LSHIFT; }

	/* NOTYET
	"++"					{ return INC_OP; }
	"--"					{ return DEC_OP; }
	"->"					{ return PTR_OP; }
	"&&"					{ return AND_OP; }
	"||"					{ return OR_OP; }
	 */

"<="		{ return LE; }
">="		{ return GE; }
"=="		{ return EQ; }
"!="		{ return NE; }

("["|"<:")	{ return '['; }
("]"|":>")	{ return ']'; }
("{"|"<%")	{ return '{'; }
("}"|"%>")	{ return '}'; }

	/* NOTYET
	"("		{ return '('; }
	")"		{ return ')'; }
	"<"		{ return '<'; }
	">"		{ return '>'; }
	"&"		{ return '&'; }
	"|"		{ return '|'; }
	"^"		{ return '^'; }
	"?"		{ return '?'; }
	":"		{ return ':'; }
	"!"		{ return '!'; }
	"~"		{ return '~'; }
	","		{ return ','; }
	"="		{ return '='; }
	"."		{ return '.'; }
	"*"		{ return '*'; }
	"+"		{ return '+'; }
	"/"		{ return '/'; }
	"%"		{ return '%'; }
	";"		{ return ';'; }
	"-"		{ return '-'; }
	 */
[()<>?:&|^!~,=.*+/%;-]	{ return *yytext; }

{WS}+		{ /* ignore whitespace */ }

.		{ yyerror("Unknown character"); }

%%

#ifdef	NOTYET
int yywrap(void)        /* called at end of input */
{
    return 1;           /* terminate now */
}

static void comment(void)
{
    int c;

    while ((c = input()) != 0)
	if (c == '*') {
	    while ((c = input()) == '*')
		;

	    if (c == '/')
		return;

	    if (c == 0)
		break;
	}
    yyerror("unterminated comment");
}

static int check_type(void)
{
    switch (sym_type(yytext)) {
    case TYPEDEF_NAME:                /* previously defined */
	return TYPEDEF_NAME;
    case ENUMERATION_CONSTANT:        /* previously defined */
	return ENUMERATION_CONSTANT;
    default:                          /* includes undefined */
	return IDENTIFIER;
    }
}

static void Qappend(struct Qcontext_t *tok, char *text)
{
    size_t len = strlen(text);
    if((len+tok->used+1)>(tok->size))
    {
        tok->size += MAX(len+1, TEXT_BLOCK_SIZE);
        tok->value = realloc(tok->value, tok->size);
        tok->value[tok->used] = '\0';
    }
    strcat(tok->value, text);
    tok->used += len;
}

static char * Qdonate(struct Qcontext_t *tok)
{ 
    char *str = tok->value;     //transfer to parser
    memset(tok, 0, sizeof(struct Qcontext_t));
    return str;
}
#endif

void Qparse_flex_init(Qparse_t *x)
{
    x->flex_extra = calloc(1, sizeof(struct Qcontext_t));
    Qyylex_init(&x->flex_scanner);
#ifdef	NOTYET
    Qyy_scan_string(x->text, x->flex_scanner);
#endif
    Qyyset_extra(x->flex_extra, x->flex_scanner);
}

void Qparse_flex_destroy(Qparse_t *x)
{
    Qyylex_destroy(x->flex_scanner);
    free(x->flex_extra);
    x->flex_extra = NULL;
}
